//--- gsParser.pas -------------------------------------------------------------
//  This is a part of the GSPackage Delphi5 components package.
//  TgsParser component unit.
//  (c) Gregory Sitnin, 2001
//------------------------------------------------------------------------------
unit gsParser;

//--- INTERFACE section --------------------------------------------------------
interface

uses
  classes, sysutils;

type
  TgsParser_ParseMode = (pmPercents, pmAmpersand, pmRaw);

  TgsParser_ScriptVariable = class
  public
    Name: string;
    Value: string;
    end;

  TgsParser = class(TComponent)
  private
    { Private declarations }
    FSourceText: TStrings;
    FParsedText: TStrings;
    FVariables: TList;
    FAutoParse: boolean;
    FParseMode: TgsParser_ParseMode;
    procedure SetSourceText(const Value: TStrings);
    function GetParsedText: TStrings;
    procedure SetParseMode(const Value: TgsParser_ParseMode);
  protected
    { Protected declarations }
  public
    { Public declarations }
    property SourceText: TStrings read FSourceText write SetSourceText;
    property ParsedText: TStrings read GetParsedText;
    function Parse: boolean;
    function DefineVariable(const VarName, VarValue : string; CanRedefine: boolean = true): boolean;
    function UndefineVariable(const VarName : string): boolean;
    function GetVariable(const VarName : string): string;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property AutoParse: boolean read FAutoParse write FAutoParse;
    property ParseMode: TgsParser_ParseMode read FParseMode write SetParseMode;
  end;

procedure Register;

//--- IMPLEMENTATION section ---------------------------------------------------
implementation

procedure Register;
begin
  RegisterComponents('GSPackage (non vis)', [TgsParser]);
end;

{ TgsParser }
constructor TgsParser.Create(AOwner: TComponent);
begin
  inherited;
  FVariables := TList.Create;
  FSourceText := TStringList.Create;
  FParsedText := TStringList.Create;
end;

destructor TgsParser.Destroy;
var i: integer;
    v: TgsParser_ScriptVariable;
begin
  inherited;
  for i := (FVariables.Count - 1) downto 0 do
  begin
    v := FVariables.Items[i];
    FVariables.Delete(i);
    v.Free;
  end;
  FVariables.Free;
  FParsedText.Free;
  FSourceText.Free;
end;

procedure TgsParser.SetSourceText(const Value: TStrings);
begin
  FSourceText.Assign(Value);
end;

function TgsParser.GetParsedText: TStrings;
begin
  if AutoParse then Parse;
  Result := FParsedText;
end;

procedure TgsParser.SetParseMode(const Value: TgsParser_ParseMode);
begin
  FParseMode := Value;
  if AutoParse then Parse;
end;

function TgsParser.DefineVariable(const VarName, VarValue: string; CanRedefine: boolean): boolean;
var v: TgsParser_ScriptVariable;
    i: integer;
    f: integer;
    vn: string;
begin
  f := -1;
  if ParseMode = pmRaw then
    vn := VarName else
    vn := UpperCase(VarName);
  for i:=0 to (FVariables.Count - 1) do
  begin
    v := FVariables.Items[i];
    if v.Name = vn then
    begin
      f := 0;
      if CanRedefine then
      begin
        v.Value := VarValue;
        f := 1;
      end;
    end;
  end;
  if (f = -1) then
  begin
    v := TgsParser_ScriptVariable.Create;
    v.Name := vn;
    v.Value := VarValue;
    FVariables.Add(v);
    f := 0;
  end;
  if (f = 0) then Result := true else Result := false;
end;

function TgsParser.UndefineVariable(const VarName: string): boolean;
var v: TgsParser_ScriptVariable;
    f: integer;
    i: integer;
    vn: string;
begin
  f := -1;
  if ParseMode = pmRaw then
    vn := VarName else
    vn := UpperCase(VarName);
  for i:=(FVariables.Count - 1) downto 0 do
  begin
    v := FVariables.Items[i];
    if v.Name = vn then
    begin
      FVariables.Delete(i);
      v.Free;
      f := 0;
    end;
  end;
  if (f = 0) then Result := true else Result := false;
end;

function TgsParser.GetVariable(const VarName: string): string;
var v: TgsParser_ScriptVariable;
    i: integer;
    vn: string;
begin
  if ParseMode = pmRaw then
    vn := VarName else
    vn := UpperCase(VarName);
  Result := '';
  for i:=0 to (FVariables.Count - 1) do
  begin
    v := FVariables.Items[i];
    if v.Name = vn then
      Result := v.Value;
  end;
end;

function TgsParser.Parse: boolean;
  //**! Begin of internal TgsParser.Parse method routines
  function ReplaceSubStr(SubString, RepString, AString: string): string;
  var s: string;
      s2: string;
      p: integer;
  begin
    s := AString;
    repeat
      if ParseMode = pmRaw then s2 := s else s2 := UpperCase(s);
      p := pos(SubString,s2);
      if p < 1 then break;
      delete(s,p,length(SubString));
      insert(RepString,s,p);
    until false;
    Result := s;
  end;
  //**! End of internal TgsParser.Parse method routines

var
  v: TgsParser_ScriptVariable;
  i: integer;
  vn: string;
  t: string;
begin
  Result := false;
  if (Result) then begin end; //**! Anti-hint
  t := FSourceText.Text;
  for i:=0 to (FVariables.Count - 1) do
  begin
    v := FVariables.Items[i];
    if (ParseMode = pmRaw) then vn := v.Name else
    if (ParseMode = pmPercents) then vn := '%'+v.Name+'%' else
    if (ParseMode = pmAmpersand) then vn := '&'+v.Name;
    t := ReplaceSubStr(vn,v.Value,t);
  end;
  FParsedText.SetText(pchar(t));
  Result := true;
end;

end.
